package org.firstinspires.ftc.teamcode.runmodes;

import static org.firstinspires.ftc.teamcode.PedroConstants.BACK_LEFT_MOTOR;
import static org.firstinspires.ftc.teamcode.PedroConstants.BACK_LEFT_MOTOR_DIRECTION;
import static org.firstinspires.ftc.teamcode.PedroConstants.BACK_RIGHT_MOTOR;
import static org.firstinspires.ftc.teamcode.PedroConstants.BACK_RIGHT_MOTOR_DIRECTION;
import static org.firstinspires.ftc.teamcode.PedroConstants.FRONT_LEFT_MOTOR;
import static org.firstinspires.ftc.teamcode.PedroConstants.FRONT_LEFT_MOTOR_DIRECTION;
import static org.firstinspires.ftc.teamcode.PedroConstants.FRONT_RIGHT_MOTOR;
import static org.firstinspires.ftc.teamcode.PedroConstants.FRONT_RIGHT_MOTOR_DIRECTION;

import com.qualcomm.robotcore.hardware.DcMotor;
import com.qualcomm.robotcore.hardware.DcMotorEx;
import com.qualcomm.robotcore.hardware.Gamepad;
import com.qualcomm.robotcore.hardware.HardwareMap;

import org.firstinspires.ftc.robotcore.external.Telemetry;
import org.firstinspires.ftc.teamcode.pedroPathing.follower.Follower;
import org.firstinspires.ftc.teamcode.pedroPathing.localization.Pose;
import org.firstinspires.ftc.teamcode.subsystem.ClawSubsystem;
import org.firstinspires.ftc.teamcode.util.action.RunAction;

public class Teleop {

    private ClawSubsystem claw;
    private Follower follower;
    private DcMotorEx leftFront;
    private DcMotorEx leftRear;
    private DcMotorEx rightFront;
    private DcMotorEx rightRear;
    private Telemetry telemetry;

    private Gamepad gamepad1;
    private Gamepad currentGamepad1;
    private Gamepad previousGamepad1;

    public Teleop(HardwareMap hardwareMap, Telemetry telemetry, Follower follower, Gamepad gamepad1) {

        claw = new ClawSubsystem(hardwareMap, telemetry);
        initMotors(hardwareMap);

        this.follower = follower;

        this.telemetry = telemetry;
        this.gamepad1 = gamepad1;

        this.currentGamepad1 = new Gamepad();
        this.previousGamepad1 = new Gamepad();
    }

    public void start() {
        claw.start();
        follower.startTeleopDrive();
    }

    public void update() {
        previousGamepad1.copy(currentGamepad1);
        currentGamepad1.copy(gamepad1);

        if (currentGamepad1.a && !previousGamepad1.a)
            claw.switchState();

        follower.setTeleOpMovementVectors(-gamepad1.left_stick_y, -gamepad1.left_stick_x, -gamepad1.right_stick_x);
        follower.update();

        telemetry.addData("X", follower.getPose().getX());
        telemetry.addData("Y", follower.getPose().getY());
        telemetry.addData("Heading", Math.toDegrees(follower.getPose().getHeading()));
        telemetry.addData("Claw State", claw.getState());
        telemetry.update();
    }

    private void initMotors(HardwareMap hardwareMap) {
        rightFront = hardwareMap.get(DcMotorEx.class, FRONT_RIGHT_MOTOR);
        rightRear = hardwareMap.get(DcMotorEx.class, BACK_RIGHT_MOTOR);
        leftFront = hardwareMap.get(DcMotorEx.class, FRONT_LEFT_MOTOR);
        leftRear = hardwareMap.get(DcMotorEx.class, BACK_LEFT_MOTOR);

        rightFront.setDirection(FRONT_RIGHT_MOTOR_DIRECTION);
        rightRear.setDirection(BACK_RIGHT_MOTOR_DIRECTION);
        leftFront.setDirection(FRONT_LEFT_MOTOR_DIRECTION);
        leftRear.setDirection(BACK_LEFT_MOTOR_DIRECTION);

        leftFront.setZeroPowerBehavior(DcMotor.ZeroPowerBehavior.BRAKE);
        leftRear.setZeroPowerBehavior(DcMotor.ZeroPowerBehavior.BRAKE);
        rightRear.setZeroPowerBehavior(DcMotor.ZeroPowerBehavior.BRAKE);
        rightFront.setZeroPowerBehavior(DcMotor.ZeroPowerBehavior.BRAKE);
    }
}
