package org.firstinspires.ftc.teamcode.subsystem;

import static org.firstinspires.ftc.teamcode.configs.RobotConstants.armBucket;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.armFloor;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.armPark;

import com.qualcomm.robotcore.hardware.HardwareMap;
import com.qualcomm.robotcore.hardware.ServoImplEx;

import org.firstinspires.ftc.robotcore.external.Telemetry;
import org.firstinspires.ftc.teamcode.util.action.Actions;
import org.firstinspires.ftc.teamcode.util.action.RunAction;

public class ArmSubsystem {

    public enum ArmState {
        PARK, FLOOR, BUCKET
    }

    private ServoImplEx arm;
    private ArmState state;
    public RunAction toFloorPosition, toParkPosition, toBucketPosition;
    private Telemetry telemetry;

    public ArmSubsystem(HardwareMap hardwareMap, Telemetry telemetry) {
        this.arm = hardwareMap.get(ServoImplEx.class, "arm-servo");
        this.toParkPosition = new RunAction(this::toParkPosition);
        this.toFloorPosition = new RunAction(this::toFloorPosition);
        this.toBucketPosition = new RunAction(this::toBucketPosition);
        this.telemetry = telemetry;
    }

    public void setState(ArmState armState) {
        if (armState == ArmState.FLOOR) {
            this.arm.setPosition(armFloor);
        } else if (armState == ArmState.PARK) {
            this.arm.setPosition(armPark);
        } else if (armState == ArmState.BUCKET) {
            this.arm.setPosition(armBucket);
        }
        this.state = armState;
    }

    public void toFloorPosition() {
        this.setState(ArmState.FLOOR);
    }

    public void toParkPosition() {
        this.setState(ArmState.PARK);
    }

    public void toBucketPosition() {
        this.setState(ArmState.BUCKET);
    }

    public void switchState() {
        if (this.state == ArmState.FLOOR) {
            this.setState(ArmState.PARK);
        } else if (this.state == ArmState.PARK) {
            this.setState(ArmState.BUCKET);
        } else if (this.state == ArmState.BUCKET) {
            this.setState(ArmState.FLOOR);
        }
    }

    public ArmState getState() {
        return this.state;
    }

    public void init() {
        this.arm.resetDeviceConfigurationForOpMode();
        Actions.runBlocking(this.toParkPosition);
    }

    public void start() {
        Actions.runBlocking(this.toParkPosition);
    }

    public double getPosition() {
        return this.arm.getPosition();
    }

    public void setPosition(double position) {
        this.arm.setPosition(position);
    }

}
