package org.firstinspires.ftc.teamcode.subsystem;

import static org.firstinspires.ftc.teamcode.configs.RobotConstants.wristBucket;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.wristFloor;

import com.qualcomm.robotcore.hardware.HardwareMap;
import com.qualcomm.robotcore.hardware.ServoImplEx;

import org.firstinspires.ftc.robotcore.external.Telemetry;
import org.firstinspires.ftc.teamcode.util.action.Actions;
import org.firstinspires.ftc.teamcode.util.action.RunAction;

public class WristSubsystem {

    public enum WristState {
        FLOOR, BUCKET
    }

    public ServoImplEx wrist;
    public WristState state;
    public RunAction toFloorPosition, toBucketPosition;
    public Telemetry telemetry;

    public WristSubsystem(HardwareMap hardwareMap, Telemetry telemetry) {
        this.wrist = hardwareMap.get(ServoImplEx.class, "wrist-servo");
        this.telemetry = telemetry;
        toBucketPosition = new RunAction(this::toBucketPosition);
        toFloorPosition = new RunAction(this::toFloorPosition);
    }

    public void setState(WristState wristState) {
        if (wristState == WristState.FLOOR) {
            wrist.setPosition(wristFloor);
        } else if (wristState == WristState.BUCKET) {
            wrist.setPosition(wristBucket);
        }
        this.state = wristState;
    }

    public void toFloorPosition() {
        setState(WristState.FLOOR);
    }

    public void toBucketPosition() {
        setState(WristState.BUCKET);
    }

    public void switchState() {
        if (state == WristState.FLOOR) {
            setState(WristState.BUCKET);
        } else if (state == WristState.BUCKET) {
            setState(WristState.FLOOR);
        }
    }

    public WristState getState() {
        return this.state;
    }

    public void init() {
        wrist.resetDeviceConfigurationForOpMode();
        Actions.runBlocking(toFloorPosition);
    }

    public void start() {
        Actions.runBlocking(toFloorPosition);
    }

    public double getPosition() {
        return this.wrist.getPosition();
    }

    public void setPosition(double position) {
        this.wrist.setPosition(position);
    }


}
