package org.firstinspires.ftc.teamcode.cometbots.poc;

import com.acmerobotics.dashboard.FtcDashboard;
import com.acmerobotics.dashboard.config.Config;
import com.acmerobotics.dashboard.telemetry.MultipleTelemetry;
import com.qualcomm.robotcore.eventloop.opmode.Autonomous;
import com.qualcomm.robotcore.eventloop.opmode.Disabled;
import com.qualcomm.robotcore.eventloop.opmode.OpMode;

import org.firstinspires.ftc.robotcore.external.Telemetry;
import org.firstinspires.ftc.teamcode.pedroPathing.follower.Follower;
import org.firstinspires.ftc.teamcode.pedroPathing.localization.Pose;
import org.firstinspires.ftc.teamcode.pedroPathing.pathGeneration.BezierCurve;
import org.firstinspires.ftc.teamcode.pedroPathing.pathGeneration.BezierLine;
import org.firstinspires.ftc.teamcode.pedroPathing.pathGeneration.PathChain;
import org.firstinspires.ftc.teamcode.pedroPathing.pathGeneration.Point;
import com.acmerobotics.roadrunner.SleepAction;
/**
 * This is the Circle autonomous OpMode. It runs the robot in a PathChain that's actually not quite
 * a circle, but some Bezier curves that have control points set essentially in a square. However,
 * it turns enough to tune your centripetal force correction and some of your heading. Some lag in
 * heading is to be expected.
 *
 * @author Anyi Lin - 10158 Scott's Bots
 * @author Aaron Yang - 10158 Scott's Bots
 * @author Harrison Womack - 10158 Scott's Bots
 * @version 1.0, 3/12/2024
 */
@Config
@Disabled
@Autonomous(name = "AsherPathV1", group = "Autonomous Pathing Tuning")
public class AsherPathV1 extends OpMode {
    private Telemetry telemetryA;

    private Follower follower;

    private PathChain path;

    private final Pose startPose = new Pose(10.0, 40, 90);

    /**
     * This initializes the Follower and creates the PathChain for the "circle". Additionally, this
     * initializes the FTC Dashboard telemetry.
     */
    @Override
    public void init() {
        follower = new Follower(hardwareMap);

        follower.setMaxPower(.4);

        follower.setStartingPose(startPose);

        path = follower.pathBuilder()
                /*
                * Only update this path
                 */
                .addPath(
                        // Line 1
                        new BezierCurve(
                                new Point(9.757, 84.983, Point.CARTESIAN),
                                new Point(33.000, 105.000, Point.CARTESIAN),
                                new Point(80.000, 118.000, Point.CARTESIAN),
                                new Point(55.000, 120.000, Point.CARTESIAN)
                        )
                )
                .addPath(
                        // Line 2
                        new BezierCurve(
                                new Point(55.000, 120.000, Point.CARTESIAN),
                                new Point(22.000, 106.000, Point.CARTESIAN),
                                new Point(11.000, 131.000, Point.CARTESIAN)
                        )
                )
                .addPath(
                        // Line 3
                        new BezierCurve(
                                new Point(11.000, 131.000, Point.CARTESIAN),
                                new Point(75.000, 95.000, Point.CARTESIAN),
                                new Point(112.000, 132.000, Point.CARTESIAN),
                                new Point(61.000, 131.000, Point.CARTESIAN)
                        )
                )
                .addPath(
                        // Line 4
                        new BezierLine(
                                new Point(61.000, 131.000, Point.CARTESIAN),
                                new Point(11.000, 131.000, Point.CARTESIAN)
                        )
                )
                .addPath(
                        // Line 5
                        new BezierCurve(
                                new Point(11.000, 131.000, Point.CARTESIAN),
                                new Point(100.000, 118.000, Point.CARTESIAN),
                                new Point(103.000, 135.000, Point.CARTESIAN),
                                new Point(61.000, 135.000, Point.CARTESIAN)
                        )
                )
                .addPath(
                        // Line 6
                        new BezierLine(
                                new Point(61.000, 135.000, Point.CARTESIAN),
                                new Point(11.000, 131.000, Point.CARTESIAN)
                        )
                )
                .addPath(
                        // Line 7
                        new BezierCurve(
                                new Point(11.000, 131.000, Point.CARTESIAN),
                                new Point(113.000, 95.000, Point.CARTESIAN),
                                new Point(67.000, 95.000, Point.CARTESIAN)
                        )
                )
                .setConstantHeadingInterpolation(Math.toRadians(90)).build();
                /*
                * End of only update this path
                 */

        follower.followPath(path);

        telemetryA = new MultipleTelemetry(this.telemetry, FtcDashboard.getInstance().getTelemetry());
        telemetryA.update();
    }

    /**
     * This runs the OpMode, updating the Follower as well as printing out the debug statements to
     * the Telemetry, as well as the FTC Dashboard.
     */
    @Override
    public void loop() {
        follower.update();
        if (follower.atParametricEnd()) {
            follower.followPath(path);
        }
        follower.telemetryDebug(telemetryA);
    }
}
